from flask import Blueprint, render_template

main_blueprint = Blueprint('main', __name__, template_folder='templates')

@main_blueprint.route('/')
def index():
    return render_template('main/index.html')

@main_blueprint.route('/stats')
def stats():
    return render_template('main/stats.html')
