{% extends "base.html" %}

{% block title %}Create Watcher{% endblock %}

{% block body %}
<form method="POST">
    {{ form.csrf_token }}
    <div class="form-group">
        {{ form.watcher_class.label }} {{ form.watcher_class(class_="form-control") }}
    </div>
    <button type="submit" class="btn btn-primary">Submit</button>
</form>
{% endblock %}
