body {
  padding-top: 5rem;
}

.content-wrapper {
  padding: 3rem 1.5rem;
  text-align: center;
}

.dropdown:hover .dropdown-menu {
    display: block;
}

.footer {
  position: absolute;
  bottom: 0;
  width: 100%;
  height: 60px; /* Set the fixed height of the footer here */
  line-height: 60px; /* Vertically center the text there */
  background-color: #f5f5f5;
}
